"use strict";

(function(window, undefined) {

  if(window.iw === undefined) window.iw = {};
  
   Vue.component('edit-assertion', {
  
    template: `
      <tr>   
        <td class="action" width="40px">
          <span v-if="editing">
            <a href="javascript:void(0)" v-bind:disabled="!isDirty" v-on:click="editing = false; assertionChanged();  " > <span class="glyphicon glyphicon-ok"     aria-hidden="true" title="save"  /></a>
            <a href="javascript:void(0)"                            v-on:click="editing = false; assertionUnchanged();" > <span class="glyphicon glyphicon-remove" aria-hidden="true" title="cancel"/></a>
           </span>
          <span v-else>
             <a href="javascript:void(0)" v-bind:disabled="editing" v-on:click="editing = true"                       > <span class="glyphicon glyphicon-pencil" aria-hidden="true" title="edit"  /></a>
             &nbsp;           
          </span> 
        </td>
        
        <td v-if="editing"> <!--path -->
            <input type="text" placeholder="path to variable"  v-model="path" v-on:keyup="isDirty=true"/>
        </td>
        <td v-else>
          {{path}}
        </td>
        <td v-if="editing"> <!-- check -->
          <select class="short" type="text" v-model='check' v-on:change="isDirty=true">
             <option v-for="c in possibleChecks">{{c}}</option>
          </select>
        </td>
        <td v-else>
            {{check}}
        </td>
        <td v-if="editing"> <!-- value -->
            <input type="text" placeholder="expected value" v-model="value" v-on:keyup="isDirty=true"/>
        </td>
        <td v-else>
            {{value}}
        </td>
        
        <td width="20px">
          <a href="javascript:void(0)"                            v-on:click="assertionRemoved();    " > <span class="glyphicon glyphicon-remove" aria-hidden="true" title="remove"/></a>
        </td>
    
          <!--
          <select v-if="possibleValues" class="short" type="text" v-model='value' v-on:change="isDirty=true">
             <option v-for="v in possibleValues">{{v}}</option>
          </select>
          <input v-else v-bind:type="inputType" v-model='value' v-on:keyup="isDirty=true"/>
           -->
        
      </tr>
    `,
    
    props: ['initialPath', 'initialCheck', 'initialValue', 'possibleChecks', 'initialFoo'],
    
    data : function() {
      iw.log("[edit-assertion] props: initialPath:" + this.initialPath + "; initialCheck: " + this.initialCheck + "; initialValue: " + this.initialValue + "; new: " + this.initialFoo, 3)
      return {
        path:        this.initialPath,
        check:       this.initialCheck,
        value:       this.initialValue,
        editing:     this.initialFoo,
        isDirty:     false
      }
    },
    
    methods : {
    
      assertionChanged : function() {
        
        //iw.log("Assertion changed; path: " + path + "; check: " + check + "; value: " + value);
        
        this.$emit('assertion-changed', {"path": this.path, "check": this.check, "value" : this.value});
        this.isDirty = false;
      },
      
      assertionUnchanged : function() {    
        if(this.initialFoo) {
            this.$emit('assertion-removed')
        }
        else {
          this.path    = this.initialPath;
          this.check   = this.initialCheck;
          this.value   = this.initialValue;
          this.isDirty = false;
        }        
      },
      
      assertionRemoved : function() {
        if(this.initialFoo)
          this.$emit('assertion-removed') 
        else
          this.$emit('assertion-removed-save');
      },
           
    }  
  });
    
    
  Vue.component('edit-stub',{
  
    template : "#edit-stub-template",
      
    props : ['initialStub', 'localPackages', 'aliases', 'baseDir', 'testCaseDir', 'testSuiteName', 'testCaseName', 'stix', 'packageName', 'distributedEnabled'],
      
    data : function() {
      iw.log("Initializing 'edit-stub'-component. testCaseDir: " + this.testCaseDir, 3);
        return {
          //make an independent copy of initialStub, so we can easily revert
          stub             : JSON.parse(JSON.stringify(this.initialStub)),
          stubTypes        : ['pipeline','service', 'exception','dynamic'],
          scopeTypes       : ['session','user','server'],
          initialStubType  : this.initialStub.pipeline ? 'pipeline' : (this.initialStub.service ? 'service' : (this.initialStub.exception ? 'exception' : (this.initialStub.dynamic ? 'dynamic' : null))),
          stubType         : this.initialStub.pipeline ? 'pipeline' : (this.initialStub.service ? 'service' : (this.initialStub.exception ? 'exception' : (this.initialStub.dynamic ? 'dynamic' : null))),
          isDirty          : false          
      }
    },
    
    computed : {
       stubBaseDir : function() {
          return this.baseDir + "/" + this.testCaseDir + "/stub-" + this.stub['service-to-stub'].replace(":","."); 
       }
    },
    
    methods: {
    
      handleStubTypeChange : function(stubType) {
        iw.log("Stub type change detected. old type: " + this.stubType + "; new type: " + stubType);
        if(stubType === 'pipeline') {
          
          if(!this.stub.pipeline) { //Create new pipeline config
            this.createPipelineStubFile();
            
          }
        }
        if(stubType === 'service') {
          //empty service, no input-file
          if(!this.stub.service) { 
            this.stub.service = {name:"",'input-file':""};   
            this.createServiceInputFile();
          }
        }
        if(stubType === 'exception') {
          if(!this.stub.exception) this.stub.exception = {"class" : "", message : ""};
        }
        
        if(stubType === 'dynamic') {
          if(!this.stub.dynamic) {
            this.createDynamicStubFiles();
            
          }
        }
        
        this.stubType = stubType;
        this.isDirty = true;
       },
      
      handleServiceToStubChange : function(newServiceName) {
        let oldServiceName = this.stub['service-to-stub'];
        iw.log("Handle Service To Stub change. Old name: " + oldServiceName + "; new name: " + newServiceName, 3)
        let self = this;
        //try to rename the stub directory
        let params = {  "base-dir"          : this.baseDir + "/" + this.testCaseDir, 
                        "existing-file-name": "stub-" + this.stub['service-to-stub'].replace(":","."),
                        "new-file-name"     : "stub-" + newServiceName.replace(":",".")
                        };
        iw.log("Params for stub directory change: " + JSON.stringify(params), 3);
        iw.invoke("iw.test.util.file:rename", params, function(data){
            if(data["$success"] === "true") {  
              //the change the paths in the stub accordingly
              self.changePathNames(oldServiceName, newServiceName);
            } else {
              iw.log("Could not rename directory of stub service "+ newServiceName );
            
            }
        }, iw.testsuites.error);  
      
        this.stub['service-to-stub'] = newServiceName;
        this.isDirty = true;
   
      },
         
      createPipelineStubFile : function () {
        if(this.stub['service-to-stub'] !== "") {
                      
          this.stub.pipeline = {"file-name" :  this.testCaseDir + "/stub-" + this.stub['service-to-stub'].replace(":",".") + "/pipeline.xml"};

          iw.log("Creating pipeline file " + this.baseDir + "/" + this.testCaseDir + "/" + this.stub.pipeline["file-name"]);

          let params = {"$document": {}, "$path" : this.stub.pipeline["file-name"], "$baseDir": this.baseDir}

          iw.invoke("iw.test.util.pub:savePipelineToFile", params, function(data){
                 iw.log("Pipeline file created? " + data["$success"], 3);
              }, iw.testsuites.error);
        } else {
          iw.log("Could not create pipeline file: 'service-to-stub' is empty");
        }
      },
         
      createServiceInputFile : function() {
        if(this.stub['service-to-stub'] !== "") {
        
          this.stub.service = {"input-file" :  this.testCaseDir + "/stub-" + this.stub['service-to-stub'].replace(":",".") + "/substitute-input.xml"};

          iw.log("Creating service input file " + this.baseDir + "/" + this.testCaseDir + "/" + this.stub.service["input-file"]);

          let params = {"$document": {}, "$path" :  this.stub.service["input-file"], "$baseDir": this.baseDir}

          iw.invoke("iw.test.util.pub:savePipelineToFile", params, function(data){
              iw.log("Service input file created? " + data["$success"], 3);
          }, iw.testsuites.error   );
        } else {
          iw.log("Could not create service input-file: 'service-to-stub' is empty");
        }         
      },
         
      createDynamicStubFiles : function() {
        if(this.stub['service-to-stub'] !== "") {
                    
            this.stub.dynamic = {"stub-base-dir" :  this.testCaseDir + "/stub-" + this.stub['service-to-stub'].replace(":",".")};

            let firstDynamicInputFile  = this.stub.dynamic['stub-base-dir'] + "/00001/test-input.xml"
            let firstDynamicOutputFile = this.stub.dynamic['stub-base-dir'] + "/00001/test-output.xml"
            this.stub.dynamic["00001"] = {"input" : firstDynamicInputFile, "output": firstDynamicOutputFile};

            iw.log("Creating dynamic file " + this.baseDir + "/" + firstDynamicInputFile);

            let params = {"$document": {}, "$path" : firstDynamicInputFile, "$baseDir": this.baseDir };

            iw.invoke("iw.test.util.pub:savePipelineToFile", params, function(data){
              iw.log("Dynamic stub input file created? " + data["$success"]);
            }, iw.testsuites.error   );

            iw.log("Creating dynamic file " + this.baseDir + "/" + firstDynamicOutputFile);
            
            params = {"$document": {}, "$path" : firstDynamicOutputFile, "$baseDir": this.baseDir};
            
            iw.invoke("iw.test.util.pub:savePipelineToFile", params, function(data){
              iw.log("Dynamic stub output file created? " + data["$success"]);
            }, iw.testsuites.error   );
        } else {
          iw.log("Could not create dynamic stub file: 'service-to-stub' is empty");
        }         
      },
         
      changePathNames : function(oldServiceName, newServiceName) {
        //pipeline['file-name']
        //service['input-file]
        //dynamic['stub-base-dir']
        if(this.stub.pipeline) {
          this.stub.pipeline['file-name'] = this.stub.pipeline['file-name'].replace(oldServiceName.replace(":","."), newServiceName.replace(":", "."));
          iw.log("Updated stub pipeline file name. New file name: " + this.stub.pipeline['file-name'], 3 );
        }
        if(this.stub.service && this.stub.service['input-file']) {
          this.service['input-file'] = this.service['input-file'].replace(oldServiceName.replace(":","."), newServiceName.replace(":", "."));
          iw.log("Updated stub service input file name. New file name: " +  this.service['input-file'], 3 );
        }
        if(this.stub.dynamic && this.stub.dynamic['stub-base-dir']) {
          this.stub.dynamic['stub-base-dir'] = this.stub.dynamic['stub-base-dir'].replace(oldServiceName.replace(":","."), newServiceName.replace(":", "."));
          iw.log("Update stub dynamic stub directory: new directory: " + this.stub.dynamic['stub-base-dir'] , 3);
        }
      },
         
      showPipelineDialog : function (baseDir, fileName, service, output, error) {
        this.$emit('show-pipeline-editor', {"baseDir": baseDir, "fileName": fileName, "service" : service, "output": output, "error": error});
      },  
             
      removeStub : function(stix) {
        //First clean up any files
        this.removeAllStubFiles()
        if(this.stub.foo) {   //the stub was in state 'initial'              
          this.$emit('remove-stub', stix)              
        } else {
          this.$emit('remove-stub-save', stix)
        }      
      },
                
      removePipelineFile : function () {
        let params = {"file-name": this.stub.pipeline['file-name'], "base-dir" : this.baseDir};
        iw.invoke("iw.test.util.file:remove", params, function(data){
            iw.log("Stub pipeline file removed? " + data["$success"]);
        }, iw.testsuites.error);
      },       
      removeServiceStubInput : function() {
        let params = {"file-name": this.stub.service['input-file'], "base-dir" : this.baseDir};
        iw.invoke("iw.test.util.file:remove", params, function(data){
            iw.log("Stub service input file removed? " + data["$success"]);
        }, iw.testsuites.error);         
      },
       
      removeDynamicStubInputFiles : function () {
        let params = {"file-name": this.stub.dynamic['stub-base-dir'], "base-dir" : this.baseDir, recursive: true, "only-subdirs": true};
        iw.invoke("iw.test.util.file:remove", params, function(data){
            iw.log("Stub dynamic files removed? " + data["$success"]);
        }, iw.testsuites.error);  
      },
       
      removeAllStubFiles : function () {
        let params = {"file-name": this.stubBaseDir, "base-dir" : ".", recursive: true};
        iw.log("Remove all stub files; params: " + JSON.stringify(params), 3);
        iw.invoke("iw.test.util.file:remove", params, function(data){
            iw.log("Stub dynamic files removed? " + data["$success"]);
        }, iw.testsuites.error);  
      },
       
      resetStub : function() {
        iw.log("Reset stub called. current stub: " + JSON.stringify(this.stub)); 
        iw.log("initial stub: " + JSON.stringify(this.initialStub));
        this.cleanupStubFiles(true);
        let self = this;
        //try to rename the stub directory if the service to stub has changed
        if(this.stub['service-to-stub'] !== this.initialStub['service-to-stub']) {
          let params = {  "base-dir"          : this.baseDir + "/" + this.testCaseDir, 
                          "existing-file-name": "stub-" + this.stub['service-to-stub'].replace(":","."),
                          "new-file-name"     : "stub-" + this.initialStub['service-to-stub'].replace(":",".")
                          };
            iw.invoke("iw.test.util.file:rename", params, function(data){
              if(data["$success"] === "true") {  
                  iw.log("Renamed stub directory back to 'stub-"+ self.initialStub['service-to-stub'] + "'");
              } else {
                iw.log("Could not rename stub directory back to "+ self.initialStub['service-to-stub'] );              
              }
          }, iw.testsuites.error);                                
        }
        this.isDirty = false;
        this.$emit('reset-stub');   
      },
       
      cleanupStubFiles : function(isReset) {
        iw.log("Clean up stub called. Reset: " + isReset,3);
        iw.log("this.stubType: " + this.stubType + "; this.initialStubType: " + this.initialStubType, 3);
        // clean up 
        if((!isReset && this.stubType  === 'pipeline') || (isReset && this.initialStubType === 'pipeline')) {
          iw.log("Cleaning up all but pipeline input", 3);
          if(this.stub.service  !== undefined) this.removeServiceStubInput();
          if(this.stub.dynamic  !== undefined) this.removeDynamicStubInputFiles();
         
          delete(this.stub.service);
          delete(this.stub.exception);
          delete(this.stub.dynamic);
          
        } else if((!isReset && this.stubType === 'service') || (isReset && this.initialStubType === 'service')) {
          iw.log("Cleaning up all but service input", 3);
          if(this.stub.pipeline !== undefined) this.removePipelineFile();
          if(this.stub.dynamic  !== undefined) this.removeDynamicStubInputFiles();
        
          delete(this.stub.pipeline);
          delete(this.stub.exception);
          delete(this.stub.dynamic);
        } else if((!isReset && this.stubType === 'exception') || (isReset && this.initialStubType === 'exception')) {
          iw.log("Cleaning up all stub files", 3);
          this.removeAllStubFiles();
          
          delete(this.stub.pipeline);
          delete(this.stub.service);
          delete(this.stub.dynamic);
          
        } else if((!isReset && this.stubType === 'dynamic') || (isReset && this.initialStubType === 'dynamic')) {
          iw.log("Cleaning up all but dynamic stub data", 3);
          if(this.stub.pipeline !== undefined) this.removePipelineFile();
          if(this.stub.service  !== undefined) this.removeServiceStubInput();            
          
          delete(this.stub.pipeline);
          delete(this.stub.service);
          delete(this.stub.exception);
          
        } else {
          iw.log("Not cleaning stub files: no condition applied", 3);
        }
      },
      
      saveStub : function() {
        this.isDirty = false;
        delete(this.stub.foo);
        this.cleanupStubFiles();
        this.$emit('save-stub', this.stub);
     }
    }    
  });
  
  Vue.component('edit-callback', {
   
    template : "#edit-callback-template",
    
    props : ['initialCallback', 'localPackages', 'aliases', 'baseDir', 'testCaseDir', 'testSuiteName', 'testCaseName', 'cbix', 'packageName', 'distributedEnabled', 'assertionTypes'],
    
    data : function() {
      return {
          //make an independent copy of initialCallback, so we can easily revert
          cb               : JSON.parse(JSON.stringify(this.initialCallback)),
          eventTypes       : ['start','success','end','error'],
          isDirty          : false,
          expectedKey      : Date.now()
      }
    },
    
    computed : {
    
    },
    
    methods : {
    
      showPipelineDialog : function (baseDir, fileName) {
            let output = this.cb.event === 'end' || this.cb.event === 'success' ;
            let error  = this.cb.event === 'error';
            this.$emit('show-pipeline-editor', {"baseDir": baseDir, "fileName": fileName, "service" : this.cb.service, "output": output, "error": error});
      },  
      
      removeCallback : function(cbix) {
        iw.log("Removing callback: cleaning up files", 3);
        if(this.cb.expected !== undefined && this.cb.expected["file-name"] !== "") {
          iw.log("Deleting file " + this.cb.expected["file-name"] + "; base-dir: " + this.baseDir, 3);
          var params = {"file-name": this.cb.expected["file-name"], "base-dir": this.baseDir};
          iw.invoke("iw.test.util.file:remove", params, function(){}, iw.testsuites.error );        
        }
        if(this.cb.foo) {
          this.$emit('remove-callback', this.cbix)
        } else {
          this.$emit('remove-callback-save', this.cbix)
        }
        
      },
      
      addAssertion: function(callback) {
        if(callback.expected === undefined){
          Vue.set(callback, "expected", {});
        }
        if(callback.expected.assertions === undefined) {
          Vue.set(callback.expected, "assertions", []);
        }
        
        callback.expected.assertions.push({"path": null, "check": "equals", "value": null, key: Date.now(), "foo": true});
      },  
      
      removeAssertion : function(expected, adx) {
         expected.assertions.splice(adx, 1);
         if(expected.assertions.length === 0) delete(expected.assertions);
      },
      
      saveCallback : function() {
          this.isDirty = false;
          delete(this.cb.foo);      //Remove the 'new' marker
          this.$emit('save-callback', this.cb);      
      },
      
      resetCallback : function() {
        let self = this;
        if(this.initialCallback.expected === undefined || this.initialCallback.expected["file-name"] === undefined) {
          if( this.cb.expected &&  this.cb.expected['file-name']) {
            iw.log("Callback reset: there was no initial expected file. Deleting "+ this.cb.expected["file-name"] , 3);
            var params = {"file-name": this.cb.expected["file-name"], "base-dir": this.baseDir};
            iw.invoke("iw.test.util.file:remove", params, function(){}, iw.testsuites.error );        
          } else {
            iw.log("Callback reset: no file to clean up", 3);
          }

        } else {
          if(this.initialCallback.expected["file-name"] === this.cb.expected["file-name"]) {
            iw.log("Callback reset called: there was no change in service or event type. Nothing to delete");
          } else {
            iw.log("Callback reset called. Renaming " + this.cb.expected["file-name"] + " to " + this.initialCallback.expected["file-name"], 3) ;
            var params = {"existing-file-name": this.cb.expected["file-name"], "new-file-name": this.initialCallback.expected["file-name"], "base-dir": this.baseDir};
              iw.invoke("iw.test.util.file:rename", params, function(result){
                if(result.$success === "true") {
                  iw.log("Callback pipeline file renamed. old name: " + params["existing-file-name"] +  "; new name: " + params["new-file-name"]);
                
                } else {
                  iw.log("Callback pipeline file rename failed: continue to use the old name: " + params["existing-file-name"] , 3);            
                }
            
              }, iw.testsuites.error );
          }
        }
        this.isDirty = false;
        this.$emit('reset-callback');
      },
      
    
      handleCallbackServiceChange : function(cb, field, value) {
        let self = this;
        iw.log("Callback changed. Dir: " + this.testCaseDir, 3);
        if(field === 'service') {
          iw.log("Old service: " + cb.service +"; new service: " + value, 3);
          //
        }
        if(field === 'event') {
          iw.log("Old event: " + cb.event +"; new event: " + value);          
        }
        if(field === 'service' || cb.service !== "") {   //both event and service are present
          
          //Compose the callback file name using use values
          var cbFileName = iw.testsuites.composeCallbackFileName(this.testCaseDir, field === 'service' ? value: cb.service, field === 'event' ? value : cb.event);                                                   
          
          if(cb.expected["file-name"] === "") {
            
            //create empty pipeline
            iw.log("Callback pipeline created. Name: " + cbFileName, 3);
            cb.expected["file-name"] = cbFileName;
            self.expectedKey++;  //rerender the 'expected component
            var params = {"$document": {}, "$path" : cb.expected["file-name"], "$baseDir": this.baseDir}
            iw.invoke("iw.test.util.pub:savePipelineToFile", params, function(){}, iw.testsuites.error   );

          } else { 
            //rename existing pipeline if we're dealing with a callback created via the UI
            
            var expectedOldCbFileName = iw.testsuites.composeCallbackFileName(this.testCaseDir, cb.service, cb.event);
            iw.log("Expected old callback file name: " + expectedOldCbFileName, 3 );
            iw.log("Current callback file name: "      + cb.expected["file-name"], 3 );
            
            if(expectedOldCbFileName ===  cb.expected["file-name"]) {
              var params = {"existing-file-name": cb.expected["file-name"], "new-file-name": cbFileName, "base-dir": this.baseDir};
              iw.log("Rename parameters: " + JSON.stringify(params));
              iw.invoke("iw.test.util.file:rename", params, function(result){
                if(result.$success === "true") {
                  iw.log("Callback pipeline file renamed. old name: " + cb.expected["file-name"] +  "; new name: " + cbFileName);

                  cb.expected["file-name"] = cbFileName;  
                  self.expectedKey++;  //rerender the 'expected component
                
                } else {
                  iw.log("Callback pipeline file rename failed: continue to use the old name: " + cb.expected["file-name"], 3);            
                }
            
              }, iw.testsuites.error );

            } else {
              iw.log("Existing callback file name was not created via the UI. Leaving it as is");
            }                                              
          }
        }
 
        if(field === 'service') cb.service = value;
        if(field === 'event')   cb.event   = value; 
      },    
    
    }
  
  });
  
  Vue.component('edit-expected',{
  
    template : "#edit-expected",
    
    props    : {
      packageName : String,
      baseDir     : String,
      testCase    : Object
    },
    
    data : function() {
      return {          
          ex               : this.testCase.expected ? JSON.parse(JSON.stringify(this.testCase.expected)) : {},
          expectedType     : this.testCase.expected && this.testCase.expected['file-name'] ? 'Expected' : this.testCase.expected && this.testCase.expected.exception && this.testCase.expected.exception.message ? 'Exception' : '',
          expectedTypes    : ['Expected','Exception', ''],
          isDirty          : false      
      }
    },
    
    methods  : {
      showPipelineDialog : function(baseDir, fileName, service, output, error) {
        this.$emit('edit-file',{baseDir : baseDir, fileName : fileName, service: service, output: output, error: error})
      },
       
      handleExpectedTypeChange : function(expectedType) {
        iw.log("Expected type change. new type: [" + expectedType + "]", 3);
        if(expectedType === 'Expected' && !this.ex['file-name'] ) {
           Vue.set(this.ex, 'file-name', "");
           Vue.set(this.ex, 'lax'      ,false);
        } else if(expectedType === 'Exception' && !this.ex.exception){
           this.ex.exception = {message: "", "class-name" : ""}        
        }
        this.expectedType = expectedType;       
      },
      
      assignExpectedFileName($event) {
          Vue.set(this.ex, 'file-name', $event);
      },
      
      saveExpected : function() {
        if(this.expectedType === '') {
          delete(this.ex['file-name']);
          delete(this.ex['lax']);
          delete(this.ex.exception);
        } else if(this.expectedType === 'Exception') {
          delete(this.ex['file-name']);
          delete(this.ex['lax']);        
        } else {
          delete(this.ex.exception);
        }
         this.isDirty = false;
         this.$emit('save-expected', this.ex)         
      },
      
      resetExpected : function() {
         
         this.isDirty = false;
         this.$emit('refresh-expected')      
      }
       
       
    
    }
  
  });
  
  iw.testsuites = new Vue({
    created()
    {
      //This piece of logic serves to decode the url:
      //split the query string into parameters and values
      let uri = window.location.href.split('?');
      if (uri.length == 2)
      {
        let vars = uri[1].split('&');
        let getVars = {};
        let tmp = '';
        vars.forEach(function(v){
          tmp = v.split('=');
          if(tmp.length == 2)
          getVars[tmp[0]] = decodeURIComponent((tmp[1] + '').replace(/\+/g, '%20'));
        });
        this.queryParams = getVars;
        iw.log(this.queryParams,3);
      }
    },
    el: '#testsuites-app',
    data : {
    
      info : {},
      
      assertionTypes: {},
      
      suites: {},
      
      filteredSuites : {},
      
      testSuiteData: {
        baseDir: "",
        packageName: "",
        service: "",
        testSuite: {}
      },

      showrundiv           : false,
      clearTestResults     : false,
      executeIndices           : [],
      remoteTestSuiteAlias : "",
      
      showexecutediv: false,
      isRepairMode  : false,
      
      packageFilter   : "",
      interfaceFilter : "",
      serviceFilter   : "",
      executeType     : "",
      remoteAlias     : "",
      running         : false,
      
      lastTestSuiteService : "",
      
      showdeletediv: false,
      deleteIndices: [],
      removeDataOnDisk: true,
      
      showpipelinediv: false,
      pipeline: {
        data                          : {},
        isproblematic                 : false,
        path                          : "",
        raw                           : "",
        service                       : null,
        schema                        : null,
        generateMinimal               : true,
        generateOnlyStructure         : false,
        generateNumberOfArrayElements : 1, 
        sampleData                    : null,
        sampleDataXML                 : null
      },
      
      showexecutesetupteardowndiv: false,
      executeServiceData: {
        remoteAlias : "",
        service     : "",
        baseDir     : "",
        fileName    : "",
        type        : ""
      },
      
      isdirtypipeline : false,
      
      pipelinePopup : null,
      
      packagesExpansionList : [],
      
      testCasesExpansionList : [],
      
      showeditdiv: false,
      
      showraweditor: false,    
      
      showresultsdiv: false,
      
      showduplicatetestcasediv : false,
      
      modal: false,
      
      duplicateTestCaseData : {
        remoteTestSuiteAlias : "",
        testSuiteServiceName : "",
        service              : "",
        originalTestCaseName : "",
        newTestCaseName      : "",
        originalStubs        : [],        
        testCaseName         : "",
        extraStubbedServices : [],
        tmpInputFile         : "",
        copyAssertions       : true,
        copyCallbacks        : true
      },
      
      results: {},
      
      selectedService: "",
      
      localPackages : [],
      
      message: "",
      
      success: true,
      
      environments: [],
      
      environment: "",

      aliases: [],      
      
      config : {}
       
    },
    computed : {
    
      runUrl : function() {
        let url = "";
        if(this.info.host) {
          url = this.info.host.protocol + "://" + this.info.host.server + ":" + this.info.host.port + "/invoke/iw.test.execution.pub/run";
          
          let query = "";
        
          if(this.executeType !== "") {
            if(query.length > 0) query += "&";
            query += "type=" + this.executeType;
          }
        
          if(this.packageFilter !== "") {
            if(query.length > 0) query += "&";
            query += "packageFilter=" + encodeURIComponent(this.packageFilter);
          }
        
          if(this.interfaceFilter !== "") {
            if(query.length > 0) query += "&";
            query += "interfaceFilter=" + encodeURIComponent(this.interfaceFilter);
          }
        
          if(this.serviceFilter !== "") {
            if(query.length > 0) query += "&";
            query += "serviceFilter=" + encodeURIComponent(this.serviceFilter);
          }
        
          if(this.remoteAlias !== "") {
            if(query.length > 0) query += "&";
            query += "remote-alias=" + encodeURIComponent(this.remoteAlias);          
          }
          if(this.environment !== "") {
            if(query.length > 0) query += "&";
            query += "environment=" + encodeURIComponent(this.environment);            
          }
          if(query !== "") url += "?" + query;
          
          this.getFilteredTestSuites()
          
        }
        iw.log("Computed url: " + url, 3);
        return url
      },
      containsCallbacks : function() {
        let result = false;
        if(this.testSuiteData.testSuite && this.testSuiteData.testSuite["test-cases"]) {
        
          if(this.executeIndices && this.executeIndices.length > 0 ) {
              for(let k = 0; k < this.executeIndices.length ; k++) {

                let tc = this.testSuiteData.testSuite["test-cases"][this.executeIndices[k]]
                 if(tc.callbacks && tc.callbacks.length > 0) {               
                    result = true;
                    break ;            
                }
              }
          } else {
            //loop over test-cases and check whether there are any valid ones
            for(let i = 0; i < this.testSuiteData.testSuite["test-cases"].length; i ++) {
                let tc = this.testSuiteData.testSuite["test-cases"][i];         
                if(tc.callbacks && tc.callbacks.length > 0) {               
                    result = true;
                    break;            
                }
            }
          }
        }
        return result;
      }
    },
    
    watch : {
      //Whenever environment changes, update the list of allowed remote aliases
      environment : function(newEnvironment, oldEnvironment) {
        if(newEnvironment !== "") {
          this.aliases = this.environments.filter(function(env){ return env.name === newEnvironment})[0].aliases
        } else {
          iw.invoke("iw.test.alias:list", {}, function(data) {
            iw.log("Aliases retrieved from server", 3);
            iw.testsuites.aliases = data.aliases.map(function(a){return a.split(":")[0]}).sort(function(a,b){return a > b});          
          });
        }
      }
    },
    
    methods : {
    
      //Use this function for showing the result of actions, not for data retrieval
      response :function(data) {
         iw.log("Logging response from IS", 4);
         iw.log(JSON.stringify(data), 4);
         iw.testsuites.showMessage(data["$message"], data["$success"] === "true" , 2500);        
         //iw.testsuites.getAllTestSuites();
      },
      
      error : function(msg) { 
        iw.log(msg, 0);
        iw.testsuites.showMessage(msg, false , 3000);       
      },
      
      showMessage : function(msg, success, timeout) {  
         iw.testsuites.message = msg;
         iw.testsuites.success = success;
         if(typeof(timeout) === "number") {
            setTimeout(function(){iw.testsuites.message= ""; iw.testsuites.success = true}, timeout);
         }
      },
      nonAlphaNumsToUnderscores : function(text) {
		      // \W matches all non-alphanumeric. Equivalent to [^A-Za-z0-9_]
		      return text.replace(/[\W]+/g, '_');
      },
      hideMessage : function() {
         iw.testsuites.message = "";
         iw.testsuites.success = true;
      },
      
      getInfo : function() {
        iw.invoke("iw.test.admin:info", {}, function(data) {
          iw.testsuites.info = data;          
        });
      },
      
      getConfig : function() {
        iw.invoke("iw.test.config:get", {}, function(data) {
          iw.testsuites.config = data;          
        });        
      },
      
      getAssertionTypes : function() {
        iw.invoke("iw.test.assertion.service:getAssertionTypes", {}, function(data) {
          iw.testsuites.assertionTypes = data['assertion-types'];          
        })      
      },
      
      refreshAllTestSuites : function() {
        iw.invoke("iw.test.ui.testsuite:list", {"refresh" : "true"}, function(data) {
          iw.testsuites.suites = data;          
          iw.log("Test suite data retrieved from server (refresh)", 3);
                
        })
      
      },
      
      getAllTestSuites : function(isInitial) {
        iw.invoke("iw.test.ui.testsuite:list", {}, function(data) {
          iw.testsuites.suites = data;          
          iw.log("Test suite data retrieved from server", 3);

          //Handle query parameters: open a test suite editor and expand the right test case          
          if(isInitial) {
            
            if(iw.testsuites.queryParams && iw.testsuites.queryParams.testSuiteName) {
              iw.log("Query params: testSuiteName: " + iw.testsuites.queryParams.testSuiteName + "; testCaseName: " + iw.testsuites.queryParams.testCaseName, 3)
              //Lookup the test suite service based on testSuiteName. The list is available in iw.testsuites.suites
              for(let i = 0 ; i < iw.testsuites.suites['test-packages'].length ; i ++) {
                for(let j = 0 ; j < iw.testsuites.suites['test-packages'][i]['test-suite-service'].length ; j ++) {
                  if(iw.testsuites.suites['test-packages'][i]['test-suite-service'][j]['test-suite-name'] === iw.testsuites.queryParams.testSuiteName ) {
                    var testSuiteName = iw.testsuites.suites['test-packages'][i]['test-suite-service'][j].name;
                    var packageName   = iw.testsuites.suites['test-packages'][i]['package'];
                    break;
                  }
                }     
              }
               
              if(testSuiteName) {
                iw.testsuites.showEditDialog(testSuiteName, packageName, iw.testsuites.queryParams.testCaseName );
                //Expand the package and highlight the service
                Vue.set(iw.testsuites.packagesExpansionList, packageName, true);
              } else {
                iw.log("No test suite service found for test suite " + iw.testsuites.queryParams.testSuiteName, 3);
              }
            }
          }          

        });
        
        iw.invoke("iw.test.environment:list", {environment: this.environment}, function(data){
          iw.testsuites.environments = data.environments;
        });
        iw.invoke("iw.test.alias:list", {}, function(data) {
          iw.log("Aliases retrieved from server", 3);
          iw.testsuites.aliases = data.aliases.map(function(a){return a.split(":")[0]}).sort(function(a,b){return a > b});         
        });
       
      },
      
      getFilteredTestSuites : function() {
        iw.invoke("iw.test.ui.testsuite:list",
          {"packageFilter": this.packageFilter, "interfaceFilter" : this.interfaceFilter, "serviceFilter": this.serviceFilter, "type": this.executeType}, 
            function(data) {
              iw.testsuites.filteredSuites = data;          
              iw.log("Filtered est suites retrieved from server", 3);
            }       
        )
      },      
      
      getTestSuite : function(testSuiteServiceName, showFunction) {
        iw.testsuites.testSuiteData.service      = testSuiteServiceName;

        iw.invoke(testSuiteServiceName, {"definition-only" : true}, function(data) {

          let now = Date.now();          
          let testCases = data["test-suite"]["test-cases"];
          //Define keys for test suites, assertions, stubs and callbacks
          for(let i = 0; i < testCases.length; i++) {
              testCases[i].key = now;
              now += 100;
              
              if(testCases[i].stubs !== undefined) {
                for(let j = 0 ; j < testCases[i].stubs.length; j++) {
                  testCases[i].stubs[j].key = now;
                  now += 100;
                }              
              }
              if(testCases[i].callbacks !== undefined) {
                  for(let k = 0 ; k < testCases[i].callbacks.length; k++) {
                    testCases[i].callbacks[k].key = now;
                    now += 100;
                } 
              }
              
              if(testCases[i].expected !== undefined) {
                if(testCases[i].expected.assertions !== undefined) {
                  for(let n = 0 ; n < testCases[i].expected.assertions.length; n++) {
                    testCases[i].expected.assertions[n].key = now;
                    now +=100;
                  }
                }
              }
            
          }

          iw.testsuites.testSuiteData.testSuite    = data["test-suite"];
          iw.testsuites.testSuiteData.baseDir      = data["base-dir"];
          
          //retrieve most recent results for this test suite, if any
          //This is used to indicate whether repairs are necessary
          let params = { "summary-only": true, "test-suite-name" : iw.testsuites.testSuiteData.testSuite.name};
          iw.invoke("iw.test.results.pub:get", params, function(data){
            iw.testsuites.results = Object.freeze(data.results);
          });
          
          if(showFunction) {showFunction()};
          iw.log("Retrieved test suite data for " + testSuiteServiceName);
        }        
        
      )},
      
      showRunDialog : function(pkg) {
        iw.testsuites.packageFilter = pkg;
        iw.testsuites.showrundiv = true;  
      },
      
      hideRunDialog : function() {
        iw.testsuites.showrundiv = false;      
      },
      
      runTestSuites : function(message) {

        iw.log("Running test suites. type: [" + this.executeType + "] package filter: [" + this.packageFilter + "] interface filter: [" + this.interfaceFilter + "] service filter: [" + this.serviceFilter + "] environment: [" + this.environment + "] remote alias: " + this.remoteAlias, 3);
        var pipeline = {
            "type"            : this.executeType,
            "packageFilter"   : this.packageFilter,
            "interfaceFilter" : this.interfaceFilter,
            "serviceFilter"   : this.serviceFilter,
            "environment"     : this.environment,
            "remote-alias"    : this.remoteAlias
        };
        var params = {
          "service" : "iw.test.execution.pub:run",
          "async"   : "true",
          "pipeline" : pipeline
        }

        iw.invoke("iw.test.execution.pub:run", pipeline, function(data){
          iw.testsuites.running = false;
          iw.testsuites.showMessage(data["$message"], data["$success"] === "true" , 5000);
        });
        if(message !== undefined) {
            iw.testsuites.message = message;      
        } else {
            iw.testsuites.message = "Running test suites..";
        }
        
        iw.testsuites.running = true;
        iw.testsuites.showrundiv = false;
      },
      

      showExecuteDialogAndRetrieve : function(testSuiteServiceName, packageName) {
        iw.log("Show execute dialog called for service: " + testSuiteServiceName, 3)
        //Only retrieve test suite if it's not already present
        if(iw.testsuites.testSuiteData.service !== testSuiteServiceName) {
          iw.testsuites.testSuiteData.packageName = packageName
          iw.testsuites.getTestSuite(testSuiteServiceName, function(){
            iw.testsuites.showexecutediv = true;
          });
        } else {
            iw.testsuites.showexecutediv = true;
        }
        
      },
      
      showExecuteDialog : function(testSuiteServiceName, repair) {
      
       iw.testsuites.isRepairMode   = repair;        
       iw.testsuites.showexecutediv = true;
      
      },
      
      hideExecuteDialog : function() {
        iw.testsuites.showexecutediv = false;   
        iw.testsuites.executeIndices = [];
      },
      
      executeInlineTestCase : function(testSuiteService, clearTestResults, index) {
        if(iw.testsuites.config["ui.distributed.enabled"] === "true") {
          this.executeIndices = [index];
          this.showExecuteDialog(testSuiteService, false);
        } else {
          this.executeTestSuite(testSuiteService, clearTestResults, [index]);
        }
      },
      
      executeTestSuite : function(testSuiteService, clearTestResults, executeIndices, remoteTestSuiteAlias, environment) {
        iw.log("Executing test suite '" + testSuiteService +"'" + executeIndices + "; remote alias: " + remoteTestSuiteAlias +"; environment: " + environment, 3)
        //Show message popup
        iw.testsuites.message = "Running test suite " + testSuiteService;
        var message = "";
        if(clearTestResults) {
          iw.invoke("iw.test.results.pub:clear", {}, function(){})
          iw.testsuites.clearTestResults = false;
          message = "Results cleared\n";
        }

        var params = {
            "index"              : executeIndices.map(i => {return ++i}).join(","), 
            "remote-alias"       : remoteTestSuiteAlias,
            "environment"        : environment,
            "expand-assertions"  : "true"
        };

        iw.invoke(testSuiteService, params, function(data){
           iw.log(data);
            
           //Sort the test suites and the test cases within
            if(data.results.testCases !== undefined) {
                
                data.results.testCases.sort(function(a,b){
                  return a.name < b.name ? -1 : 1;                  
                });
                
                data.results.testCases.map(function(tc){
                  
                  if(tc.assertions !== undefined) {
                  
                    tc.assertions.sort(function(a,b){
                      return a.name < b.name ? -1 : 1;
                    });                  
                  
                  }
                });
                
            }
                      
           iw.testsuites.showexecutediv = false;
           iw.testsuites.results        = Object.freeze(data.results);
           iw.testsuites.showresultsdiv = true;
           iw.testsuites.executeIndices = [];
           iw.testsuites.message        = "";
           iw.testsuites.isRepairMode   = false;
        }, function(message){
          
          iw.testsuites.showMessage(message, false, 3000);
          iw.testsuites.showexecutediv = false;
        });        
      },
    
      isTestCaseFailed : function(testSuiteName, testCaseName) {
        let c = 0;
        
        if(iw.testsuites.results.name === testSuiteName) {
           c = iw.testsuites.results.testCases.reduce(function(sum, tc){
            if(tc.name.split("::")[0] === testCaseName && tc.result === false){ return ++sum } else { return sum }
          }, 0);
        }
        return c >= 1;  
      },    
      
      repairTestSuite : function(testSuiteServiceName, index) {
        if(this.config["ui.distributed.enabled"] === "true") {
          if(index >= 0) this.executeIndices = [index];
          this.showExecuteDialog(testSuiteServiceName, true) 
        } else {
          this.doRepairTestSuite(testSuiteServiceName, [index]);
        }
      },
      
      doRepairTestSuite : function(testSuiteServiceName, executeIndices, remoteTestSuiteAlias, environment) {
        iw.log("Repairing test suite service: " + testSuiteServiceName + "; index: " + executeIndices.join(","));
        var params = {
          "repair"       : "true",
          "remote-alias" : remoteTestSuiteAlias,
          "environment"  : environment,
          "index"        : executeIndices.map(i => {return ++i}).join(","), 
        };
      
        iw.invoke(testSuiteServiceName, params, function(data) {
            iw.log("RepairTestSuite results");
            iw.log(data)
            //Refresh test suite data.
            iw.testsuites.getTestSuite(iw.testsuites.testSuiteData.service);
            
            if(executeIndices.length === 0) { 
              iw.testsuites.executeTestSuite(testSuiteServiceName, true, [], remoteTestSuiteAlias, environment);
            } else {
              iw.testsuites.executeTestSuite(testSuiteServiceName, true, executeIndices, remoteTestSuiteAlias, environment);
            }
            if(executeIndices.length === 0) {
              iw.testsuites.showMessage("Repaired test cases in test suite '" + iw.testsuites.testSuiteData.testSuite.name +"'", true, 3000);            
            } else {
              for (let i = 0; i < executeIndices.length ; i++) {
                iw.testsuites.showMessage("Repaired test case '"+ iw.testsuites.testSuiteData.testSuite['test-cases'][executeIndices[i]].name +"' in test suite '" + iw.testsuites.testSuiteData.testSuite.name +"'", true, 3000);            
              }
            }
      
          },
          function(message) {
            iw.testsuites.showMessage(message, false, 3000);
            iw.testsuites.showexecutediv = false;
          }
      
        );
        
      },
      
      showDeleteDialog : function(testSuiteServiceName, deleteIndices) {
        if(deleteIndices) {
           iw.testsuites.deleteIndices = deleteIndices;
        }
        iw.testsuites.testSuiteData.service     = testSuiteServiceName;
        iw.testsuites.getTestSuite(testSuiteServiceName);
        iw.testsuites.showdeletediv = true;      
      },
      
      hideDeleteDialog : function() {
        iw.testsuites.deleteIndices = [];
        iw.testsuites.showdeletediv = false;
      },
      
      deleteTestSuite: function(testSuiteServiceName, deleteIndices) {
        var params = {"test-suite-service"  : testSuiteServiceName, 
                      "remove-data-on-disk" : iw.testsuites.removeDataOnDisk,
                      "index" : deleteIndices.join(",")}
                        
        iw.invoke("iw.test.ui.testsuite:remove", params, function(data) {
            iw.testsuites.response(data); iw.testsuites.getAllTestSuites()
          }, iw.testsuites.error); 
        iw.testsuites.showdeletediv = false;
        iw.testsuites.showeditdiv   = false;
        iw.testsuites.deleteIndices = [];
      },
      
      deleteTestCase : function(testSuiteServiceName, index) {
        this.hideEditDialog()
        this.showDeleteDialog(testSuiteServiceName, [index]);
      },
      
      showEditDialog : function(testSuiteServiceName, packageName, testCaseName) {
        iw.log("Show edit dialog called", 3);
        iw.testsuites.lastTestSuiteService = testSuiteServiceName;        
        iw.testsuites.testSuiteData.packageName = packageName;
        iw.testsuites.listLocalPackages();
        iw.testsuites.getTestSuite(testSuiteServiceName, function(){
          iw.testsuites.showeditdiv = true;
          iw.testsuites.testCasesExpansionList =iw.testsuites.testSuiteData.testSuite["test-cases"].map(function(){return false});
          if(testCaseName) {
            let index = -1;
            for(let i =0 ; i < iw.testsuites.testSuiteData.testSuite["test-cases"].length ; i++) {
              iw.log("Considering test case " + i, 3)          
              if(iw.testsuites.testSuiteData.testSuite["test-cases"][i].name === testCaseName) {
                  iw.log("Found the test case to expand", 3)
                  index = i;
                  break;
              }
        }
        if(index >= 0) {
            iw.testsuites.toggleExpandTestCase(index);
        }
     }
        });
      },
      
      hideEditDialog : function() {
        iw.testsuites.showeditdiv = false;                      
        iw.testsuites.hidePipelineDialog();
        iw.testsuites.testCasesExpansionList = [];
      },
      
      showDuplicateTestCaseDialog : function(testSuiteServiceName, testCase) {
        //Initialize parameters
        iw.testsuites.duplicateTestCaseData.testSuiteServiceName = testSuiteServiceName;
        iw.testsuites.duplicateTestCaseData.originalTestCaseName = testCase.name;
        iw.testsuites.duplicateTestCaseData.service              = testCase.service;
        iw.testsuites.duplicateTestCaseData.originalStubs        = testCase.stubs !== undefined ? testCase.stubs.map(function(stub){return stub['service-to-stub']}) : [];        
        iw.testsuites.duplicateTestCaseData.newTestCaseName      = testCase.name + " (duplicate)";
        iw.testsuites.duplicateTestCaseData.copyAssertions       = testCase.expected.assertions && testCase.expected.assertions.length > 0
        iw.testsuites.duplicateTestCaseData.copyCallbacks        = testCase.callbacks && testCase.callbacks.length > 0;
        iw.testsuites.duplicateTestCaseData.tmpInputFile         = "tmp-"+Date.now().toString() + ".xml"
        
        if(testCase.input && testCase.input["file-name"]) {
          //There is an input file (normally the case)
          //Copy the input of this testCase to $base-dir/
        
          let params = {"base-dir": iw.testsuites.testSuiteData.baseDir, "source-file-name" : testCase.input["file-name"], "target-file-name" : iw.testsuites.duplicateTestCaseData.tmpInputFile};
        
          iw.invoke("iw.test.util.file:copy", params, function(data){
        
            iw.testsuites.showduplicatetestcasediv = true;
        
          },iw.testsuites.error);
        
        } else {
          //There is no input file, so generate an empty one
             let params = {"$document": {}, "$path" : this.duplicateTestCaseData.tmpInputFile, "$baseDir": this.testSuiteData.baseDir, "overwrite" : false}
             iw.invoke("iw.test.util.pub:savePipelineToFile", params, function(data){
                 iw.log("Setup input file created? " + data["$success"], 3);
                 iw.testsuites.showduplicatetestcasediv = true;
              }, iw.testsuites.error);              
        }
        
        
      },
      
      addExtraStubbedService : function(service) {
        
        iw.testsuites.duplicateTestCaseData.extraStubbedServices.push(service);
      },
      
      removeExtraStubbedService : function(idx) {
        iw.testsuites.duplicateTestCaseData.extraStubbedServices.splice(idx, 1);        
      },
      resetDuplicateTestCaseData : function() {        
        let params = {"base-dir" : iw.testsuites.testSuiteData.baseDir, "file-name" : iw.testsuites.duplicateTestCaseData.tmpInputFile}
        iw.invoke("iw.test.util.file:remove", params, function(data){}, iw.testsuites.error);
        iw.testsuites.duplicateTestCaseData.remoteTestSuiteAlias = "";
        iw.testsuites.duplicateTestCaseData.testSuiteServiceName = "";
        iw.testsuites.duplicateTestCaseData.originalTestCaseName = "";
        iw.testsuites.duplicateTestCaseData.newTestCaseName      = "";
        iw.testsuites.duplicateTestCaseData.originalStubs        = [];        
        iw.testsuites.duplicateTestCaseData.originalTestCaseName = "";
        iw.testsuites.duplicateTestCaseData.extraStubbedServices = [];
        iw.testsuites.duplicateTestCaseData.copyAssertions       = true;
        iw.testsuites.duplicateTestCaseData.copyCallbacks        = true;
        iw.testsuites.duplicateTestCaseData.tmpInputFile         = "";
      },
      
      duplicateTestCase : function() {     
        let params = {
          "remote-alias"            : iw.testsuites.duplicateTestCaseData.remoteTestSuiteAlias,
          "test-suite-service-name" : iw.testsuites.duplicateTestCaseData.testSuiteServiceName,
          "original-test-case-name" : iw.testsuites.duplicateTestCaseData.originalTestCaseName,
          "new-test-case-name"      : iw.testsuites.duplicateTestCaseData.newTestCaseName,          
          "action"                  : "duplicate",
          "extra-stubbed-services"  : iw.testsuites.duplicateTestCaseData.extraStubbedServices,
          "copy-callbacks"          : iw.testsuites.duplicateTestCaseData.copyCallbacks,
          "copy-inline-assertions"  : iw.testsuites.duplicateTestCaseData.copyAssertions,
          "remote-alias"            : iw.testsuites.duplicateTestCaseData.alias,
          "alternative-input-file"  : iw.testsuites.duplicateTestCaseData.tmpInputFile
        }
        iw.invoke("iw.test.generate.pub:duplicateTestCase", params, function(data){        
            iw.testsuites.response(data);
            iw.testsuites.getTestSuite(iw.testsuites.testSuiteData.service);
            iw.testsuites.resetDuplicateTestCaseData();
        },
          iw.testsuites.error);      

        iw.testsuites.showduplicatetestcasediv = false;           
      },
      
      hideDuplicateTestCaseDialog : function() {
        iw.testsuites.resetDuplicateTestCaseData();
        iw.testsuites.showduplicatetestcasediv = false;      
      },
      
      assignInputFileName : function(tc, fileName) {
        if(!tc.input) tc.input = {};
        tc.input['file-name'] = fileName; 
      },
      
      assignExpectedFileName : function(tc, fileName) {
        if(!tc.expected) tc.expected = {};
        tc.expected['file-name'] = fileName; 
      },      
      
      toggleExpandPackage : function(name) {
        iw.log("Toggle visibility package ["+name+"]", 3);
        Vue.set(iw.testsuites.packagesExpansionList, name, !iw.testsuites.packagesExpansionList[name]);
      },
      
      toggleExpandTestCase : function(index) {
        iw.log("Toggle visibility testcase ["+index+"]", 3);
        Vue.set(iw.testsuites.testCasesExpansionList, index, !iw.testsuites.testCasesExpansionList[index]);
      },
      
      showResultsModal : function() {
        iw.testsuites.showresultsdiv = true;  
      },
      
      hideResultsModal : function() {
        iw.testsuites.showresultsdiv = false;      
      },      
      
      saveTestSuite : function() {
        iw.log("About to save test suite", 3);
        //TODO: Call service to remove files superfluous callback files.
        var params   = {"test-suite-service": iw.testsuites.testSuiteData.service, 
                        "test-suite"        : iw.testsuites.testSuiteData.testSuite,
                        "base-dir"          : iw.testsuites.testSuiteData.baseDir}
               
        iw.invoke("iw.test.ui.testsuite:save", params, function(data){        
            iw.testsuites.response(data);
              iw.testsuites.getTestSuite(iw.testsuites.testSuiteData.service);
            },
          iw.testsuites.error);
                   
      },
      
      saveTestSuiteAndClose: function() {
        iw.testsuites.saveTestSuite();
        iw.testsuites.hideEditDialog();        
      },
      
      showPipelineDialog : function(baseDir, path, service, output, error) {
        iw.testsuites.getPipeline(baseDir, path, service, output, error);
      },
      
      hidePipelineDialog : function() {
        iw.testsuites.showpipelinediv = false;
        if(iw.xmleditor) iw.xmleditor.getSession().setValue("" , 1);
        if(iw.peditor)   iw.peditor.set({})
      },
      
      showJSONEditor : function() {
        iw.testsuites.showraweditor = false;
      },
      
      showXMLEditor : function() {
        iw.testsuites.showraweditor = true;
      },
  
      setPipelinePopup : function(data) {
            iw.log("Setting pipeline popup..", 3);
            this.pipelinePopup = data;
      },

      getPipeline : function(baseDir, path, service, output, error) {
        iw.testsuites.pipeline.path    = path;
        iw.testsuites.pipeline.service = service;
        
        var params = {"$path": path, "$baseDir": baseDir, "$include-raw": "true"}
        iw.invoke("iw.test.util.pub:restorePipelineFromFile", params, function(data) {
          //iw.testsuites.pipeline.data = JSON.stringify(data['$document'])
          iw.testsuites.pipeline.isproblematic = ("true" === data['$problematic']);
          
          if(iw.peditor === undefined) {
            var container = document.getElementById("pipelineeditor");
            var options = { modes:["tree","code","text"],
                            enableTransform: false,
                            onChange: function() {iw.testsuites.isdirtypipeline = true},
                            onClassName: function(c) {
                              //iw.log("on-class-name: " + c.path + " " + c.field + " " + c.value, 3)
                            }
                          }

             iw.peditor = new JSONEditor(container, options);
          }
          if(iw.xmleditor === undefined) {
             iw.xmleditor = JSONEditor.ace.edit("xmleditor");
             iw.xmleditor.getSession().setMode("ace/mode/xml");
             iw.xmleditor.on("change", function() {iw.log("XML changed",3); iw.testsuites.isdirtypipeline = true});                       
          }

          //Raw data 
          //iw.testsuites.pipeline.raw = data['$raw'];
          iw.xmleditor.getSession().setValue( data['$raw'] , 1);

          //JSON
          iw.peditor.set(data['$document'])
          iw.peditor.setName("pipeline");
         
          iw.testsuites.getPipelineSchema(false, output, error)
          //This seems to be problematic. Sometimes 'expandAll() does not exist
          //iw.peditor.expandAll();
          
          //Reset isdirtypipeline, so the save button becomes disabled
          iw.testsuites.isdirtypipeline = false;

          //Explicitly show the XML editor if there are data types present that are not
          //supported by JSON
          if( iw.testsuites.pipeline.isproblematic) {
            iw.testsuites.showXMLEditor();
          }

          //iw.testsuites.showraweditor = iw.testsuites.pipeline.isproblematic;
          iw.testsuites.showpipelinediv = true;

        }, function(msg){
            iw.testsuites.hidePipelineDialog();
            iw.testsuites.error(msg)
          })
      },

      togglePipelineValidation : function() {
        if(this.pipeline.validate) {
          iw.peditor.setSchema(this.pipeline.schema);
        } else {
          iw.peditor.setSchema(null);
        }
      },

      getPipelineSchema : function(generateData, output, error) {
          let self = this;
          if(this.pipeline.service !== null) {
             let params = { 'nodeName'                     : this.pipeline.service,
                            'generate-minimal'             : this.pipeline.generateMinimal, 
                            'generate-only-structure'      : this.pipeline.generateOnlyStructure,
                            'generate-nr-of-array-elements': this.pipeline.generateNumberOfArrayElements} 

              if(error) {
                  params.nodeName = "iw.test.doc:exception";
                  output          = false;
              }
              iw.invoke("iw.test.ui.schema:get", params, function(data){
                 if(output) { 
                   self.pipeline.schema        = data['output-schema']
                   self.pipeline.sampleData    = data['sample-output-data']
                   self.pipeline.sampleDataXML = data['sample-output-xml']
                 } else { 
                   self.pipeline.schema        = data['input-schema']
                   self.pipeline.sampleData    = data['sample-input-data']
                   self.pipeline.sampleDataXML = data['sample-input-xml']
                 }
                 if(iw.testsuites.pipeline.validate === true) {
                   iw.peditor.setSchema(self.pipeline.schema);
                 }
                 if(generateData) {
                   iw.peditor.set(self.pipeline.sampleData);
                   iw.xmleditor.getSession().setValue(self.pipeline.sampleDataXML  , 1);
                 }

              }, iw.testsuites.error)
          }
     },
      
      savePipeline : function(closeDialog) {
        var params = {"$path"    : iw.testsuites.pipeline.path, 
                      "$baseDir" : iw.testsuites.testSuiteData.baseDir,
                      "$package" : iw.testsuites.testSuiteData.packageName}
                      
        if(iw.testsuites.showraweditor) {
          params["$raw"] = iw.xmleditor.getValue();
        } else {
          params["$document"] = iw.peditor.get();
        }
        
        iw.invoke("iw.test.util.pub:savePipelineToFile", params, function(data){
          iw.testsuites.showMessage(data["$message"], data["$success"] === "true" , 2500);          
          if(closeDialog) {
            iw.testsuites.hidePipelineDialog();        
          } else {
            iw.testsuites.getPipeline(iw.testsuites.testSuiteData.baseDir, iw.testsuites.pipeline.path);       
          }
          
        }, iw.testsuites.error)
        
      },
      
      addAssertion: function(testCase) {
        iw.log("Adding new assertion to test case " + testCase.name, 3)
        if(!testCase.expected) {
          iw.log("Adding new 'expected' section to test case "+ testCase.name , 3);
          Vue.set(testCase, 'expected', {});
        }
        if(testCase.expected.assertions === undefined) {
          iw.log("Adding new 'assertion' array section to test case "+ testCase.name , 3);
          Vue.set(testCase.expected, "assertions", []);
        }
        
        testCase.expected.assertions.push({"path": null, "check": "equals", "value": null, key: Date.now(), "foo": true});
      },
      
      removeAssertion: function(expected, index) {
        expected.assertions.splice(index, 1);
        if(expected.assertions.length === 0 ) {
          iw.log("Last assertion removed: removing assertion structure", 3)
          delete(expected.assertions);
        }
      },
      
      addStub : function(testCase) {
        iw.log("Add stub called",3);
        if(testCase.stubs === undefined) {
          Vue.set(testCase, "stubs", []);
        }
        testCase.stubs.push({"service-to-stub":"", scope: "session", key: Date.now(), "foo" : "bar"});
      },
      
      removeStub : function(tc, index) {
        iw.log("Deleting stub["+index+"]; base-dir: " + iw.testsuites.testSuiteData.baseDir, 3);
        iw.log(tc.stubs[index], 3);       
        tc.stubs.splice(index, 1);
        if(tc.stubs.length === 0 ) delete(tc.stubs)
      },      
      
      setStubFromComponent : function(stubs, stix, $event) {
        iw.log("Receiving stub data from component:");
        iw.log($event);
        stubs[stix] = $event;
        iw.log("After assignment of stub to position in stubs array:");
        iw.log(iw.testsuites.testSuiteData.testSuite);
        iw.testsuites.saveTestSuite();
      },
      
      addCallback : function(testCase) {
        iw.log("Add callback called", 3);
        if(testCase.callbacks === undefined) {
          Vue.set(testCase, "callbacks", []);
        }     
        testCase.callbacks.push({service: "", event: "start", expected: {"file-name": "", lax: "true" }, key: Date.now(), "foo" : "bar"});
      },
      
      removeCallback : function(tc, index, testCaseDir) {
        tc.callbacks.splice(index, 1);
        
        if(tc.callbacks.length === 0 ) { //remove the 'callbacks' directory
          delete(tc.callbacks);
          let params = {"file-name": testCaseDir + "/callbacks", "base-dir" : iw.testsuites.testSuiteData.baseDir, "recursive": true};
             iw.invoke("iw.test.util.file:remove", params, function(data){
                  iw.log("Callbacks dir removed? " + JSON.stringify(params) + ": "+ data["$success"]);
             }, iw.testsuites.error);
        }
      },
      
      setCallbackFromComponent : function(callbacks, cbix, $event) {
        iw.log("Receiving callback data from component:");
        iw.log($event);
        callbacks[cbix] = $event;
        iw.log("After assignment of callback to position in stubs array:");
        iw.log(iw.testsuites.testSuiteData.testSuite, 3);
        iw.testsuites.saveTestSuite();
      },      
      
      setCallbackService($event, cb) {
        iw.log("Set callback service called. Service: " +$event,3);
      },
      
      assignSetupService : function(service) {
        if(!service || service ==='') {
          this.removeSetupService();
        } else {      
          if(iw.testsuites.testSuiteData.testSuite.setup === undefined) {
             let inputFileName = "setup-input.xml";
             Vue.set(iw.testsuites.testSuiteData.testSuite, 'setup', {service: service});
             
             //Create an empty input file for the setup service
             let params = {"$document": {}, "$path" : inputFileName, "$baseDir": this.testSuiteData.baseDir, "overwrite" : false}
             iw.invoke("iw.test.util.pub:savePipelineToFile", params, function(data){
                 iw.log("Setup input file created? " + data["$success"], 3);
              }, iw.testsuites.error);    
             
          } else {
             iw.testsuites.testSuiteData.testSuite.setup.service = service
          }        
        }
      },
      
      assignSetupInputFile : function(fileName) {
        if(iw.testsuites.testSuiteData.testSuite.setup !== undefined) {
           Vue.set(iw.testsuites.testSuiteData.testSuite.setup, 'file-name', fileName);
        }        
      },
      
      assignTeardownService : function(service) {
        if(!service || service ==='') {
          this.removeTeardownService();
        } else {
          if(iw.testsuites.testSuiteData.testSuite.teardown === undefined) {
             let inputFileName = "teardown-input.xml";
             Vue.set(iw.testsuites.testSuiteData.testSuite, 'teardown', {service: service});
             
             //Create an empty input file for the teardown service
             let params = {"$document": {}, "$path" : inputFileName, "$baseDir": this.testSuiteData.baseDir, "overwrite" : false}
             iw.invoke("iw.test.util.pub:savePipelineToFile", params, function(data){
                 iw.log("Teardown input file created? " + data["$success"], 3);
              }, iw.testsuites.error);      
              
          } else {
             iw.testsuites.testSuiteData.testSuite.teardown.service = service
          }       
        }
      },
      
      assignTeardownInputFile : function(fileName) {
        if(iw.testsuites.testSuiteData.testSuite.teardown !== undefined) {
           Vue.set(iw.testsuites.testSuiteData.testSuite.teardown, 'file-name', fileName);
        }        
      },
      
      showExecuteServiceDialog : function (service, baseDir, fileName, type) {
      
        if(this.config["ui.distributed.enabled"]) {
          this.executeServiceData.service   = service;
          this.executeServiceData.baseDir   = baseDir;
          this.executeServiceData.fileName  = fileName,
          this.executeServiceData.type      = type;
          this.showexecutesetupteardowndiv = true;
        } else {
          this.executeService(service, baseDir, fileName, type)  
        }
      
      },
      
      hideExecuteServiceDialog : function () {
      
        this.showexecutesetupteardowndiv = false;
      
      },
      executeService : function(service, baseDir, fileName, type, alias) {
      
        //type = [setup|teardown]
        let params = { "service": service, "base-dir": baseDir, "file-name": fileName, type: type}
        if(alias) params["remote-alias"] = alias;

        iw.invoke("iw.test.execution.service:executeSetupOrTeardown", params, function(data){
          //show results
          iw.testsuites.pipelinePopup = data.pipeline;
          iw.testsuites.showexecutesetupteardowndiv = false;
        }
        //standard error handler
        , iw.testsuites.error
      )},
      
      removeSetupService : function() {
        iw.log('Remove Setup called', 3);
        delete(iw.testsuites.testSuiteData.testSuite.setup);
      },
      
      removeTeardownService: function() {
        iw.log('Remove teardown called', 3)
        delete(iw.testsuites.testSuiteData.testSuite.teardown);
      },
      
      composeCallbackFileName : function (testCaseDir, service, event) {
        return testCaseDir + "/callbacks/" +  service.replace(":",".") + "-" +event + ".xml"
      },

      listLocalPackages : function(alias) {
		   var params = {};
		   iw.invoke("iw.test.ui.ns:listPackages", params, function(data) {
		      iw.log("Retrieved local packages", 3);

            iw.testsuites.localPackages = data.packages.filter(function(a){                 
                 var result = a["system-package"] !== "true" && a.enabled === "true";                 
                 return result;
            }).sort(function(a,b){ if(a.name >= b.name) return 1; else return -1});
            
                
         }, iw.testsuites.error);
		  }
      
    }
   
  });  
  
  iw.testsuites.getInfo();
  iw.testsuites.getConfig();
  iw.testsuites.getAssertionTypes();
  iw.testsuites.getAllTestSuites(true); 
  //iw.testsuites.listLocalPackages();


})(this, undefined);
